#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlabel.h>

#include "profile_switch.hh"
#include "profile_manager.hh"
#include "pixmaps.hh"

ProfileSwitch::ProfileSwitch (QWidget* parent, ProfileManager* manager)
  : QHBox (parent)
{
  m_manager = manager;
  self_induced = false;
  active_mark = ::GetDot ();

  QHBoxLayout* l = (QHBoxLayout*) layout ();
  l ->  addStretch (0);
  QLabel* profile_label = new QLabel (this);
  profile_label -> setText (i18n("show profile:"));
  l -> insertSpacing (2,20);
  config_selection = new QComboBox (this);
  activator = new QPushButton(i18n("activate"), this);

  slotRebuildList ();

  QObject::connect ( m_manager, SIGNAL (changedActiveProfile ()), this, SLOT (slotNewActive ()) );
  QObject::connect ( m_manager, SIGNAL (changedConfigProfile ()), this, SLOT (slotNewConfig ()) );
  QObject::connect ( m_manager, SIGNAL (newProfileList ()), this, SLOT (slotRebuildList ()) );
  QObject::connect ( config_selection, SIGNAL (activated (int)), this, SLOT (slotConfigSelected (int)) );
  QObject::connect ( activator, SIGNAL (clicked ()), this, SLOT (slotToggleActive ()) );

  activator -> setFocus ();
}

void ProfileSwitch::slotRebuildList ()
{
  std::cout << "rebuild" << std::endl;
  while (config_selection -> count () > 0)
    config_selection -> removeItem (config_selection -> count () -1);
  unsigned int profile_count = m_manager -> Profiles().size();
  int active_index = m_manager -> ActiveProfileIndex ();
  for (unsigned int i = 0; i < profile_count; i++) {
    if ((int) i == active_index)
      config_selection -> insertItem ( *active_mark, m_manager -> getI18NProfileName(i));
    else
      config_selection -> insertItem (m_manager -> getI18NProfileName(i)); 
  }
  slotNewConfig ();
}

void ProfileSwitch::slotNewConfig ()
{
  std::cout << "new config" << std::endl;
  if (!self_induced)
    config_selection -> setCurrentItem (m_manager -> ConfigProfileIndex ());
  SetButtonLabel ();
}

void ProfileSwitch::slotNewActive ()
{
  std::cout << "new active" << std::endl;
  // TODO : optimize this (change only the two relevant items !)
  unsigned int profile_count = m_manager -> Profiles().size();
  int active_index = m_manager -> ActiveProfileIndex ();
  for (unsigned int i = 0; i < profile_count; i++) {
    if ((int) i == active_index)
      config_selection -> changeItem ( *active_mark, m_manager -> getI18NProfileName(i), (int) i);
    else
      config_selection -> changeItem (m_manager -> getI18NProfileName(i), (int) i); 
  }
  SetButtonLabel ();
}

void ProfileSwitch::slotConfigSelected (int index)
{
  std::cout << "config selected by user" << std::endl;
  if (index >= 0) {
    self_induced = true;
    m_manager -> ConfigureProfile ((unsigned int) index);
    self_induced = false;
  }
  SetButtonLabel ();
}

void ProfileSwitch::slotToggleActive ()
{
  // TODO: check for modifications ? (maybe the profile_manager facilities are enough)
  int index = config_selection -> currentItem ();
  if (index >= 0) {
    if (index == m_manager -> ActiveProfileIndex ())
      m_manager -> DeactivateProfile ();
    else
      m_manager -> ActivateProfile ((unsigned int) index);
  }
}

void ProfileSwitch::SetButtonLabel ()
{
  int index = config_selection -> currentItem ();
  if (index == m_manager -> ActiveProfileIndex ())
    activator -> setText (i18n("deactivate"));
  else
    activator -> setText (i18n("activate"));
}
