#include <string>

typedef unsigned int ppp_state;
typedef unsigned int ppp_error;

namespace ppp_states {
  const ppp_state no_connection = 0;
  const ppp_state connecting = 1;
  const ppp_state connected = 2;
  const ppp_state terminated = 3;
}

namespace ppp_errors {
  const ppp_error no_error = 0;

  const ppp_error no_dialtone = 2;
  const ppp_error busy = 3;
  const ppp_error timeout = 4; // <- communication with the modem device, _not_ the logical connection
  //TODO: distinguish between time_out types:
  // const ppp_error no_modem = 1;
  // const ppp_error no_answer = 5;
  const ppp_error authentification = 6;
  const ppp_error hangup = 7;
  const ppp_error unknown = 8;
}

class PPPStatus
{
public:
  PPPStatus (const std::string& logfile_name);
  ~PPPStatus ();

  void Reset ();
  ppp_state GetStatus ();
  ppp_error GetError () { return error; }
  unsigned int GetConnectionTime ();
private:
  bool KeyWordAction (const std::string& line, const char* kw, ppp_state next, bool setnext, bool n_cmp = false);
  bool KeyWordError (const std::string& line, const char* kw, ppp_error res_error, bool n_cpm = false);

  void UpdateStatus ();
  void UpdateStatusInternal ();


  std::string log_name;
  time_t old_mod_date;
  off_t old_size;
  time_t connection_start;

  ppp_state current;
  ppp_error error;

  bool connected;
  unsigned int last_log_pos;
  unsigned int search_bound;
};


class ReverseFile
{
public:
  ReverseFile (const std::string& file_name, unsigned int low_bound = 0);
  ~ReverseFile ();

  bool bof () {return (file_pos == 0 && next_line_start == 0);}
  unsigned int FilePos () {return file_pos + next_line_start + offset;}

  std::string PrevString ();
  std::string operator() () { return PrevString ();}
private:
  std::string suffix;

  std::string CopyBuff (unsigned int lend);
  void PrevBuffer ();


  FILE* f;
  char* buffer;
  unsigned int file_pos;  // real file pos - offset
  unsigned int next_line_start;
  unsigned int offset;  // set to lower bound
};
