#include "systeminfo.hh"
#include <iostream>

int Execute (std::string& output, const std::string& cmd)
{ // TODO: sth more sophisticated...
  char tempfile[]="/tmp/netconf_XXXXXX\0";
  int descriptor = mkstemp (tempfile);
  close (descriptor);

  std::string exec=cmd + " > " + (char *) tempfile;
  // std::cout << exec << std::endl;

  output = (char*) tempfile;
  return system (exec.c_str ());
}


void CleanUp (const std::string& output_file)
{
  std::string exec="rm -f " + output_file;
  system (exec.c_str ());
}

std::string ExecuteSimple (const std::string& cmd)
{
  std::string filename;
  std::string ret="";
  int res=Execute (filename, cmd);
  if (res == 0) {
    std::ifstream f(filename.c_str());
    if (f && !f.eof ())
      std::getline (f, ret, '\n');
  }
  //  std::cout << "x" <<  ret << "x"  << std::endl;
  CleanUp (filename);
  return ret;
}


void DetectEtherDevices (EtherDevList& list)
{
  std::string filename;
  int ret = Execute (filename,
		     "/sbin/ip -o link | grep 'link/ether' | cut -d ' ' -f 2 ");
  if (ret != 0) {
    std::cerr << "warning, could not get list of ethernet devices" << std::endl;
  } else {
    
    std::ifstream f(filename.c_str ());
    if (!f)
      std::cerr << "warning: no output from 'ip link'" << std::endl;
    else {
      std::string line;
      while (!f.eof()) {
	EtherDev dev;
	std::getline(f, line, '\n');
	std::string::size_type end= line.find_first_of (":");
	dev.dev = line.substr (0, end);
	if (dev.dev != "") {

	  dev.info = devInfo (dev);
	  dev.wireless = isWlan (dev);

	  list.push_back (dev);
	}
      }      
    }

  }

  CleanUp (filename);
}

static bool inList (const SerialDevList& list, const std::string& dev)
{
  for (unsigned int i = 0; i < list.size (); i++)
    if (list[i].dev == dev)
      return true;
  return false;
}

void AddSerialUnique (SerialDevList& list, const std::string& device)
{
  SerialDev dev;
  if (!inList (list, device)) {
    dev.dev = device;
    list.push_back (dev);
  }
}


void DetectSerialDevices (SerialDevList& list)
{
  //  list.clear ();
  std::string filename;
  int ret = Execute (filename, "find `find /dev/ -type d -a -name 'tts'` -type c");
  if (ret != 0) {
    std::cerr << "warning, could not get list of serial devices" << std::endl;
  } else {
    std::ifstream f(filename.c_str ());
    if (!f)
      std::cerr << "warning: no output from 'find'" << std::endl;
    else {
      std::string line;
      while (!f.eof()) {
	std::getline(f, line, '\n');
	if (line != "")
	  AddSerialUnique (list, line);
      }
    }

  }
  CleanUp (filename);
}

void QuerySerPnP (SerialDev& dev)
{
  std::string cmd = "sudo /sbin/netconf-helper.sh serpnp " + dev.dev; 
  std::string result = ExecuteSimple(cmd);
  if (result != "") {
    std::cout << "found " << result << " attached to " << dev.dev << std::endl;
    dev.attached = result;
  }
}

void ScanSerialDevices (SerialDevList& list)
{
  for (unsigned int i=0; i<list.size(); i++)
    QuerySerPnP (list[i]);
}



bool isWlan (EtherDev& dev)
{
  std::string cmd="( /usr/sbin/iwconfig "
    + dev.dev + " 2>&1 ) | grep 'no wireless extensions.'";
  return ExecuteSimple (cmd) == "";
}

std::string devInfo (EtherDev& dev)
{
  std::string cmd = "sudo /sbin/netconf-helper.sh devinfo " + dev.dev;
  return ExecuteSimple (cmd);
}


bool ConfigfileHandleForReading (std::ifstream& stream)
{
  stream.open ("/etc/conf/network-netconf-copy");
  return (stream);
}

bool ConfigfileHandleForWriting (std::ofstream& stream)
{
  stream.open ("/etc/conf/network-netconf-copy");
  return (stream);
}

bool WriteConfig ()
{
  return system ("sudo /sbin/netconf-helper.sh");
}

void InitConfig ()
{
  system ("sudo /sbin/netconf-helper.sh init");
}


bool isUp (const std::string& interface, std::string* profile)
{
  std::string cmd = "grep " + interface + " /var/run/rocknet/active-interfaces | sed 's,.*(\\([^)]*\\)).*,\\1,g'";
  std::string p = ExecuteSimple (cmd);
  if (p != "") {
    if (profile)
      *profile = p;
    return true;
  }
  return false;
}


int WirelessLinkQuality (const std::string& device)
{
  std::string cmd = "/usr/sbin/iwconfig " + device + " 2>/dev/null "\
    " | grep 'Link Quality' | sed 's,.*Quality[:=]\\([0-9]*\\)/\\([0-9]*\\) .*,\\1 \\2,'";
  std::string result=ExecuteSimple (cmd);
  int c;
  int t;
  if (sscanf (result.c_str (), "%d %d", &c, &t) != 2 || t == 0)
    return 0;
  return (c * 100) / t;
}


int TriggerIfUp (const std::string& interface, const std::string& profile)
{
  std::string cmd = "sudo /sbin/netconf-helper.sh ifup " + interface + " " + profile;
  return system (cmd.c_str());
}

int TriggerIfDown (const std::string& interface, const std::string& profile)
{
  std::string cmd = "sudo /sbin/netconf-helper.sh ifdown " + interface + " " + profile;
  return system (cmd.c_str ());
}
